import React__default, { useState, createElement, useContext, createContext } from 'react';
import isPropValid from '@emotion/is-prop-valid';

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

var htmlTags = ['a', 'abbr', 'address', 'applet', 'area', 'article', 'aside', 'audio', 'b', 'base', 'basefont', 'bdi', 'bdo', 'blockquote', 'body', 'br', 'button', 'canvas', 'caption', 'cite', 'code', 'col', 'colgroup', 'data', 'datalist', 'dd', 'del', 'details', 'dfn', 'dialog', 'dir', 'div', 'dl', 'dt', 'em', 'embed', 'fieldset', 'figcaption', 'figure', 'font', 'footer', 'form', 'frame', 'frameset', 'h1', 'h2', 'h3', 'h4', 'h5', 'h6', 'head', 'header', 'hgroup', 'hr', 'html', 'i', 'iframe', 'img', 'input', 'ins', 'kbd', 'label', 'legend', 'li', 'link', 'main', 'map', 'mark', 'marquee', 'menu', 'meta', 'meter', 'nav', 'noscript', 'object', 'ol', 'optgroup', 'option', 'output', 'p', 'param', 'picture', 'pre', 'progress', 'q', 'rp', 'rt', 'ruby', 's', 'samp', 'script', 'section', 'select', 'slot', 'small', 'source', 'span', 'strong', 'style', 'sub', 'summary', 'sup', 'table', 'tbody', 'td', 'template', 'textarea', 'tfoot', 'th', 'thead', 'time', 'title', 'tr', 'track', 'u', 'ul', 'var', 'video', 'wbr'];

var media = ['sm', 'md', 'lg', 'xl'];
var modifiers = ['hover', 'active', 'focus'];
var append = function append(strToAppend) {
  return function (classNames) {
    return classNames.map(function (item) {
      return item.split(' ').map(function (i) {
        return strToAppend + ":" + i;
      }).join(' ');
    }).join(' ');
  };
};
var _if = function _if(prop, stylesA, stylesB) {
  if (stylesB === void 0) {
    stylesB = [];
  }

  return prop ? stylesA.join(' ') : stylesB.join(' ');
};
var _modifiers = /*#__PURE__*/modifiers.reduce(function (acc, next) {
  var _extends2;

  return _extends({}, acc, (_extends2 = {}, _extends2[next] = append(next), _extends2));
}, {});
var _media = /*#__PURE__*/media.reduce(function (acc, next) {
  var _extends3;

  return _extends({}, acc, (_extends3 = {}, _extends3[next] = append(next), _extends3));
}, {});
var _theme = function _theme(theme) {
  return function (themeData) {
    return (themeData[theme] || ['']).join(' ');
  };
};
var helpers = /*#__PURE__*/_extends({
  "if": _if
}, _modifiers, {}, _media);
var createHelpers = function createHelpers(theme) {
  return _extends({
    theme: _theme(theme)
  }, helpers);
}; // export type Helpers = typeof helpers;

var ThemeContext = /*#__PURE__*/createContext({});
function ThemeProvider(props) {
  var _React$useState = useState(props.defaultTheme),
      theme = _React$useState[0],
      setTheme = _React$useState[1];

  var changeTheme = function changeTheme(theme) {
    setTheme(theme);
  };

  return createElement(ThemeContext.Provider, {
    value: {
      theme: theme,
      changeTheme: changeTheme
    }
  }, props.children);
}
function useTheme() {
  var _React$useContext = useContext(ThemeContext),
      theme = _React$useContext.theme,
      changeTheme = _React$useContext.changeTheme;

  return {
    theme: theme,
    changeTheme: changeTheme
  };
}

function createComponent(createHelpers) {
  return function (htmlTag) {
    return function (fn) {
      return React__default.forwardRef(function (props, ref) {
        var _props$className;

        var _React$useContext = React__default.useContext(ThemeContext),
            theme = _React$useContext.theme;

        var className = [].concat(fn(_extends({}, createHelpers(theme)), props), [(_props$className = props.className) !== null && _props$className !== void 0 ? _props$className : '']).filter(function (item) {
          return item !== '';
        }).join(' ');
        var validProps = Object.keys(props).reduce(function (acc, next) {
          var _extends2;

          return isPropValid(next) ? _extends({}, acc, (_extends2 = {}, _extends2[next] = props[next], _extends2)) : acc;
        }, {});

        var _props = _extends({}, validProps, {
          ref: ref,
          className: className
        });

        return React__default.createElement(htmlTag, _props);
      });
    };
  };
}

var componentFactory = /*#__PURE__*/createComponent(createHelpers);
var components = /*#__PURE__*/htmlTags.reduce(function (acc, htmlTag) {
  var _extends3;

  return _extends({}, acc, (_extends3 = {}, _extends3[htmlTag] = componentFactory(htmlTag), _extends3));
}, {});

var tw = /*#__PURE__*/_extends({}, components, {
  macro: function macro(fn) {
    return fn(helpers);
  }
});

export { ThemeProvider, tw, useTheme };
//# sourceMappingURL=react.esm.js.map
